@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
 
-#define _GNU_SOURCE
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -35,11 +34,12 @@
 #include <ofono/modem.h>
 #include <ofono/voicecall.h>
 
+#include <drivers/atmodem/atutil.h>
+
 #include "gatchat.h"
 #include "gatresult.h"
 
 #include "common.h"
-#include "ifxmodem.h"
 
 static const char *none_prefix[] = { NULL };
 static const char *xlema_prefix[] = { "+XLEMA:", NULL };
@@ -1009,8 +1009,7 @@ static void ifx_voicecall_remove(struct ofono_voicecall *vc)
 {
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 
-	g_slist_foreach(vd->calls, (GFunc) g_free, NULL);
-	g_slist_free(vd->calls);
+	g_slist_free_full(vd->calls, g_free);
 
 	g_strfreev(vd->en_list);
 
@@ -1020,8 +1019,7 @@ static void ifx_voicecall_remove(struct ofono_voicecall *vc)
 	g_free(vd);
 }
 
-static struct ofono_voicecall_driver driver = {
-	.name			= "ifxmodem",
+static const struct ofono_voicecall_driver driver = {
 	.probe			= ifx_voicecall_probe,
 	.remove			= ifx_voicecall_remove,
 	.dial			= ifx_dial,
@@ -1041,12 +1039,4 @@ static struct ofono_voicecall_driver driver = {
 	.send_tones		= ifx_send_dtmf
 };
 
-void ifx_voicecall_init(void)
-{
-	ofono_voicecall_driver_register(&driver);
-}
-
-void ifx_voicecall_exit(void)
-{
-	ofono_voicecall_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(voicecall, ifxmodem, &driver)
