@@ -61,6 +61,7 @@
 #include <ofono/gnss.h>
 #include <ofono/handsfree.h>
 #include <ofono/siri.h>
+#include <ofono/sim-auth.h>
 
 #include <drivers/atmodem/vendor.h>
 #include <drivers/atmodem/atutil.h>
@@ -160,9 +161,7 @@ static void phonesim_activate_primary(struct ofono_gprs_context *gc,
 		break;
 	}
 
-	if (ctx->apn)
-		snprintf(buf + len, sizeof(buf) - len - 3, ",\"%s\"",
-				ctx->apn);
+	snprintf(buf + len, sizeof(buf) - len - 3, ",\"%s\"", ctx->apn);
 
 	/* Assume always succeeds */
 	if (g_at_chat_send(gcd->chat, buf, none_prefix, NULL, NULL, NULL) == 0)
@@ -411,29 +410,32 @@ static void phonesim_query_available_rats(struct ofono_radio_settings *rs,
 }
 
 static struct ofono_gprs_context_driver context_driver = {
-	.name			= "phonesim",
 	.probe			= phonesim_context_probe,
 	.remove			= phonesim_context_remove,
 	.activate_primary	= phonesim_activate_primary,
 	.deactivate_primary	= phonesim_deactivate_primary,
 };
 
+OFONO_ATOM_DRIVER_BUILTIN(gprs_context, phonesim, &context_driver)
+
 static struct ofono_radio_settings_driver radio_settings_driver = {
-	.name			= "phonesim",
 	.probe			= phonesim_radio_settings_probe,
 	.remove			= phonesim_radio_settings_remove,
 	.query_rat_mode		= phonesim_query_rat_mode,
 	.query_available_rats	= phonesim_query_available_rats,
 };
 
+OFONO_ATOM_DRIVER_BUILTIN(radio_settings, phonesim, &radio_settings_driver)
+
 static struct ofono_ctm_driver ctm_driver = {
-	.name			= "phonesim",
 	.probe			= phonesim_ctm_probe,
 	.remove			= phonesim_ctm_remove,
 	.query_tty		= phonesim_ctm_query,
 	.set_tty		= phonesim_ctm_set,
 };
 
+OFONO_ATOM_DRIVER_BUILTIN(ctm, phonesim, &ctm_driver)
+
 static int phonesim_probe(struct ofono_modem *modem)
 {
 	struct phonesim_data *data;
@@ -802,6 +804,7 @@ static void phonesim_set_online(struct ofono_modem *modem, ofono_bool_t online,
 				set_online_cb, cbd, g_free) > 0)
 		return;
 
+	g_free(cbd);
 	CALLBACK_WITH_FAILURE(cb, user_data);
 }
 
@@ -860,6 +863,7 @@ static void phonesim_post_sim(struct ofono_modem *modem)
 		ofono_sms_create(modem, 0, "atmodem", data->chat);
 
 	ofono_radio_settings_create(modem, 0, "phonesim", data->chat);
+	ofono_sim_auth_create(modem);
 }
 
 static void phonesim_post_online(struct ofono_modem *modem)
@@ -1148,10 +1152,6 @@ static int phonesim_init(void)
 
 	ofono_modem_driver_register(&localhfp_driver);
 
-	ofono_gprs_context_driver_register(&context_driver);
-	ofono_ctm_driver_register(&ctm_driver);
-	ofono_radio_settings_driver_register(&radio_settings_driver);
-
 	if (conf_override)
 		parse_config(conf_override);
 	else
@@ -1173,11 +1173,6 @@ static void phonesim_exit(void)
 	g_slist_free(modem_list);
 	modem_list = NULL;
 
-	ofono_radio_settings_driver_unregister(&radio_settings_driver);
-	ofono_ctm_driver_unregister(&ctm_driver);
-
-	ofono_gprs_context_driver_unregister(&context_driver);
-
 	ofono_modem_driver_unregister(&phonesim_driver);
 }
 
